from django.conf import settings

from lti_provider.models import LTICourseContext
from pylti.common import (
    LTIException, LTINotInSessionException, LTI_SESSION_KEY,
    verify_request_common, LTIRoleException, LTI_ROLES, LTI_PROPERTY_LIST)
from xml.etree import ElementTree as etree


LTI_PROPERTY_LIST_EX = getattr(settings,
                               'LTI_PROPERTY_LIST_EX',
                               [
                                   'custom_canvas_user_login_id',
                                   'context_title',
                                   'lis_course_offering_sourcedid',
                                   'custom_canvas_api_domain'
                               ])

LTI_PROPERTY_USER_USERNAME = getattr(settings,
                                     'LTI_PROPERTY_USER_USERNAME',
                                     'custom_canvas_user_login_id')


class LTI(object):
    """
    The LTI Object represents an abstraction of the current LTI
    session. It provides callback methods and methods that allow the
    developer to inspect LTI basic-launch-request.

    This object is instantiated by the LTIMixin.
    """

    def __init__(self, request_type, role_type):
        self.request_type = request_type
        self.role_type = role_type

    def clear_session(self, request):
        """
        Invalidate the session
        """
        request.session.flush()

    def initialize_session(self, request, params):
        # All good to go, store all of the LTI params into a
        # session dict for use in views
        for prop in LTI_PROPERTY_LIST:
            if params.get(prop, None):
                request.session[prop] = params[prop]

        for prop in LTI_PROPERTY_LIST_EX:
            if params.get(prop, None):
                request.session[prop] = params[prop]

    def verify(self, request):
        """
        Verify if LTI request is valid, validation
        depends on arguments

        :raises: LTIException
        """
        if self.request_type == 'session':
            self._verify_session(request)
        elif self.request_type == 'initial':
            self._verify_request(request)
        elif self.request_type == 'any':
            self._verify_any(request)
        else:
            raise LTIException('Unknown request type')

        return True

    def _params(self, request):
        if request.method == 'POST':
            return dict(request.POST.items())
        else:
            return dict(request.GET.items())

    def _verify_any(self, request):
        """
        Verify that request is in session or initial request.
        Guess what type of request is being sent over based on
        the request params.

        :raises: LTIException
        """

        # if an oauth_consumer_key is present, assume this is an initial
        # launch request and complete a full verification
        # otherwise, just check the session for the LTI_SESSION_KEY
        params = self._params(request)
        if 'oauth_consumer_key' in params:
            self._verify_request(request)
        else:
            self._verify_session(request)

    @staticmethod
    def _verify_session(request):
        """
        Verify that session was already created

        :raises: LTIException
        """
        if not request.session.get(LTI_SESSION_KEY, False):
            raise LTINotInSessionException('Session expired or unavailable')

    def _verify_request(self, request):
        """
        Verify LTI request

        :raises: LTIException is request validation failed
        """
        try:
            params = self._params(request)
            verify_request_common(self.consumers(),
                                  request.build_absolute_uri(),
                                  request.method, request.META,
                                  params)

            self._validate_role()

            self.clear_session(request)
            self.initialize_session(request, params)
            request.session[LTI_SESSION_KEY] = True
            return True
        except LTIException:
            self.clear_session(request)
            request.session[LTI_SESSION_KEY] = False
            raise

    def consumers(self):
        """
        Gets consumer's map from config
        :return: consumers map
        """
        config = getattr(settings, 'PYLTI_CONFIG', dict())
        consumers = config.get('consumers', dict())
        return consumers

    def _validate_role(self):
        """
        Check that user is in accepted/specified role

        :exception: LTIException if user is not in roles
        """
        if self.role_type != u'any':
            if self.role_type in LTI_ROLES:
                role_list = LTI_ROLES[self.role_type]

                # find the intersection of the roles
                roles = set(role_list) & set(self.user_roles())
                if len(roles) < 1:
                    raise LTIRoleException('Not authorized.')
            else:
                raise LTIException("Unknown role {}.".format(self.role_type))

        return True

    def custom_course_context(self, request):
        """
        Returns the custom LTICourseContext id as provided by LTI

        throws: KeyError or ValueError or LTICourseContext.DoesNotExist
        :return: context -- the LTICourseContext instance or None
        """
        return LTICourseContext.objects.get(
            enable=True,
            uuid=self.custom_course_context(request))

    def canvas_domain(self, request):
        return request.session.get('custom_canvas_api_domain', None)

    def consumer_user_id(self, request):
        return "%s-%s" % \
            (self.oauth_consumer_key(request), self.user_id(request))

    def course_context(self, request):
        return request.session.get('context_id', None)

    def course_title(self, request):
        return request.session.get('context_title', None)

    def is_administrator(self, request):
        return 'administrator' in request.session.get('roles', '').lower()

    def is_instructor(self, request):
        roles = request.session.get('roles', '').lower()
        return 'instructor' in roles or 'staff' in roles

    def lis_outcome_service_url(self, request):
        return request.session.get('lis_outcome_service_url', None)

    def lis_result_sourcedid(self, request):
        return request.session.get('lis_result_sourcedid', None)

    def oauth_consumer_key(self, request):
        return request.session.get('oauth_consumer_key', None)

    def user_email(self, request):
        return request.session.get('lis_person_contact_email_primary', None)

    def user_fullname(self, request):
        name = request.session.get('lis_person_name_full', None)
        if not name or len(name) < 1:
            name = self.user_id(request)

        return name or ''

    def user_id(self, request):
        return request.session.get('user_id', None)

    def user_identifier(self, request):
        return request.session.get(LTI_PROPERTY_USER_USERNAME, None)

    def user_roles(self, request):  # pylint: disable=no-self-use
        """
        LTI roles of the authenticated user

        :return: roles
        """
        roles = request.session.get('roles', None)
        if not roles:
            return []
        return roles.lower().split(',')

    def sis_course_id(self, request):
        return request.session.get('lis_course_offering_sourcedid', None)

    def generate_request_xml(self, message_identifier_id, operation,
                             lis_result_sourcedid, score, launch_url):
        # pylint: disable=too-many-locals
        """
        Generates LTI 1.1 XML for posting result to LTI consumer.

        :param message_identifier_id:
        :param operation:
        :param lis_result_sourcedid:
        :param score:
        :return: XML string
        """
        root = etree.Element(u'imsx_POXEnvelopeRequest',
                             xmlns=u'http://www.imsglobal.org/services/'
                                   u'ltiv1p1/xsd/imsoms_v1p0')

        header = etree.SubElement(root, 'imsx_POXHeader')
        header_info = etree.SubElement(header, 'imsx_POXRequestHeaderInfo')
        version = etree.SubElement(header_info, 'imsx_version')
        version.text = 'V1.0'
        message_identifier = etree.SubElement(header_info,
                                              'imsx_messageIdentifier')
        message_identifier.text = message_identifier_id
        body = etree.SubElement(root, 'imsx_POXBody')
        xml_request = etree.SubElement(
            body, '%s%s' % (operation, 'Request'))
        record = etree.SubElement(xml_request, 'resultRecord')

        guid = etree.SubElement(record, 'sourcedGUID')

        sourcedid = etree.SubElement(guid, 'sourcedId')
        sourcedid.text = lis_result_sourcedid
        if score is not None:
            result = etree.SubElement(record, 'result')
            result_score = etree.SubElement(result, 'resultScore')
            language = etree.SubElement(result_score, 'language')
            language.text = 'en'
            text_string = etree.SubElement(result_score, 'textString')
            text_string.text = score.__str__()
            if launch_url:
                result_data = etree.SubElement(result, 'resultData')
                lti_launch_url = etree.SubElement(
                    result_data, 'ltiLaunchUrl')
                lti_launch_url.text = launch_url
        ret = "<?xml version='1.0' encoding='utf-8'?>\n{}".format(
            etree.tostring(root, encoding='utf-8').decode('utf-8'))

        return ret



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('lti_provider', '0005_auto_20171009_1234'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lticoursecontext',
            name='lms_course_context',
            field=models.CharField(max_length=255, unique=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0006_require_contenttypes_0002'),
    ]

    operations = [
        migrations.CreateModel(
            name='LTICourseContext',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False,
                                        auto_created=True, primary_key=True)),
                ('lms_context_id', models.TextField()),
                ('faculty_group', models.ForeignKey(
                    on_delete=models.CASCADE,
                    related_name='course_faculty_group', to='auth.Group')),
                ('group', models.ForeignKey(
                    on_delete=models.CASCADE,
                    related_name='course_group',
                    to='auth.Group')),
            ],
        ),
    ]



from django.contrib.sessions.middleware import SessionMiddleware
from django.test.client import RequestFactory
from django.test.testcases import TestCase
from lti_provider.lti import LTI
from lti_provider.tests.factories import BASE_LTI_PARAMS, CONSUMERS, \
    generate_lti_request
from pylti.common import LTI_SESSION_KEY, LTINotInSessionException


class LTITest(TestCase):

    def setUp(self):
        self.request = RequestFactory()
        self.request.COOKIES = {}
        middleware = SessionMiddleware()
        middleware.process_request(self.request)
        self.request.session.save()

        for prop, value in BASE_LTI_PARAMS.items():
            self.request.session[prop] = value

        self.lti = LTI('initial', 'any')

        self.emptyRequest = RequestFactory()
        self.emptyRequest.COOKIES = {}
        middleware = SessionMiddleware()
        middleware.process_request(self.emptyRequest)
        self.emptyRequest.session.save()

    def test_init(self):
        self.assertEquals(self.lti.request_type, 'initial')
        self.assertEquals(self.lti.role_type, 'any')

    def test_consumer_user_id(self):
        self.request.session['oauth_consumer_key'] = '1234567890'
        self.assertEquals(
            self.lti.consumer_user_id(self.request), '1234567890-student')

    def test_user_email(self):
        self.assertIsNone(self.lti.user_email(self.emptyRequest))
        self.assertEquals(self.lti.user_email(self.request), 'foo@bar.com')

    def test_user_fullname(self):
        self.assertEquals(self.lti.user_fullname(self.emptyRequest), '')

        self.assertEquals(self.lti.user_fullname(self.request), 'Foo Bar Baz')

    def test_user_roles(self):
        self.assertEquals(self.lti.user_roles(self.emptyRequest), [])

        self.assertEquals(self.lti.user_roles(self.request), [
            u'urn:lti:instrole:ims/lis/instructor',
            u'urn:lti:instrole:ims/lis/staff'])

        self.assertTrue(self.lti.is_instructor(self.request))
        self.assertFalse(self.lti.is_administrator(self.request))

    def test_consumers(self):
        with self.settings(PYLTI_CONFIG={'consumers': CONSUMERS}):
            self.assertEquals(self.lti.consumers(), CONSUMERS)

    def test_params(self):
        factory = RequestFactory()
        request = factory.post('/', {'post': 'data'})
        params = self.lti._params(request)
        self.assertTrue('post' in params)

        request = factory.post('/', {'get': 'data'})
        params = self.lti._params(request)
        self.assertTrue('get' in params)

    def test_verify_any(self):
        lti = LTI('any', 'any')
        request = generate_lti_request()

        with self.settings(PYLTI_CONFIG={'consumers': CONSUMERS}):
            # test_verify_request
            lti.verify(request)
            self.assertTrue(request.session[LTI_SESSION_KEY])

            # test_verify_session
            self.assertTrue(lti.verify(request))

    def test_verify_session(self):
        lti = LTI('session', 'any')
        request = RequestFactory().post('/lti/')

        with self.assertRaises(LTINotInSessionException):
            request.session = {}
            lti.verify(request)

        request.session = {LTI_SESSION_KEY: True}
        self.assertTrue(lti.verify(request))

    def test_verify_request(self):
        with self.settings(PYLTI_CONFIG={'consumers': CONSUMERS}):
            request = generate_lti_request()
            lti = LTI('initial', 'any')
            lti.verify(request)
            self.assertTrue(request.session[LTI_SESSION_KEY])



from django.contrib import admin
from lti_provider.models import LTICourseContext


@admin.register(LTICourseContext)
class AssetAdmin(admin.ModelAdmin):
    class Meta:
        model = LTICourseContext

    list_display = ('group', 'faculty_group')



from django import template
from lti_provider.lti import LTI
from pylti.common import LTINotInSessionException

register = template.Library()


@register.simple_tag
def lti_session(request):
    try:
        lti = LTI('session', 'any')
        if lti.verify(request):
            return lti
    except LTINotInSessionException:
        return None



try:
    from urllib.parse import parse_qs, urlparse, urlencode
except ImportError:
    from urllib import urlencode
    from urlparse import parse_qs, urlparse

from django.contrib.auth.models import User, Group, AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.test.client import RequestFactory
import factory
import oauthlib.oauth1
from oauthlib.oauth1.rfc5849 import CONTENT_TYPE_FORM_URLENCODED

from lti_provider.models import LTICourseContext


BASE_LTI_PARAMS = {
    u'launch_presentation_return_url': u'/asset/',
    u'lis_person_contact_email_primary': u'foo@bar.com',
    u'lis_person_name_full': u'Foo Bar Baz',
    u'lis_result_sourcedid': u'course-v1%3AedX%2BDemoX%2BDemo_Course'
                             u':-724d6c2b5fcc4a17a26b9120a1d463aa:student',
    u'lti_message_type': u'basic-lti-launch-request',
    u'lti_version': u'LTI-1p0',
    u'roles':
        u'urn:lti:instrole:ims/lis/Instructor,urn:lti:instrole:ims/lis/Staff',
    u'resource_link_id': u'-724d6c2b5fcc4a17a26b9120a1d463aa',
    u'user_id': u'student',
}

CONSUMERS = {
    '__consumer_key__': {'secret': '__lti_secret__'}
}


def generate_lti_request(course_context=None, provider=None, use=None):
    """
    This code generated valid LTI 1.0 basic-lti-launch-request request
    """
    client = oauthlib.oauth1.Client('__consumer_key__',
                                    client_secret='__lti_secret__',
                                    signature_method=oauthlib.oauth1.
                                    SIGNATURE_HMAC,
                                    signature_type=oauthlib.oauth1.
                                    SIGNATURE_TYPE_QUERY)

    params = BASE_LTI_PARAMS.copy()
    if course_context:
        params.update({'context_id': course_context.lms_course_context})
    if provider:
        params.update({'tool_consumer_info_product_family_code': provider})
    if use:
        params.update({'ext_content_intended_use': use})

    signature = client.sign(
        'http://testserver/lti/',
        http_method='POST', body=urlencode(params),
        headers={'Content-Type': CONTENT_TYPE_FORM_URLENCODED})

    url_parts = urlparse(signature[0])
    query_string = parse_qs(url_parts.query, keep_blank_values=True)
    verify_params = dict()
    for key, value in query_string.items():
        verify_params[key] = value[0]

    params.update(verify_params)

    request = RequestFactory().post('/lti/', params)

    middleware = SessionMiddleware()
    middleware.process_request(request)
    request.session.save()

    request.user = AnonymousUser()
    return request


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
    username = factory.Sequence(lambda n: 'user%d' % n)
    password = factory.PostGenerationMethodCall('set_password', 'test')


class GroupFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Group
    name = factory.Sequence(lambda n: 'group %s' % n)


class LTICourseContextFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = LTICourseContext

    group = factory.SubFactory(GroupFactory)
    faculty_group = factory.SubFactory(GroupFactory)
    lms_course_context = factory.Sequence(lambda n: 'lti%d' % n)



from django.conf.urls import url

from lti_provider.views import LTIConfigView, LTILandingPage, LTIRoutingView, \
    LTICourseEnableView, LTIPostGrade, LTIFailAuthorization, LTICourseConfigure


urlpatterns = [
    url(r'^config.xml$', LTIConfigView.as_view(), {}, 'lti-config'),
    url(r'^auth$', LTIFailAuthorization.as_view(), {}, 'lti-fail-auth'),
    url(r'^course/config$',
        LTICourseConfigure.as_view(), {}, 'lti-course-config'),
    url(r'^course/enable/$',
        LTICourseEnableView.as_view(), {}, 'lti-course-enable'),
    url(r'^landing/$', LTILandingPage.as_view(), {}, 'lti-landing-page'),
    url('^grade/$', LTIPostGrade.as_view(), {}, 'lti-post-grade'),
    url(r'^$', LTIRoutingView.as_view(), {}, 'lti-login'),
    url(r'^assignment/(?P<assignment_name>.*)/(?P<pk>\d+)/$',
        LTIRoutingView.as_view(), {}, 'lti-assignment-view'),
    url(r'^assignment/(?P<assignment_name>.*)/$',
        LTIRoutingView.as_view(), {}, 'lti-assignment-view')
]



from django.test.testcases import TestCase
from lti_provider.lti import LTI
from lti_provider.templatetags.lti_utils import lti_session
from lti_provider.tests.factories import CONSUMERS, generate_lti_request


class LTITemplateTags(TestCase):

    def test_lti_session(self):
        with self.settings(PYLTI_CONFIG={'consumers': CONSUMERS}):
            request = generate_lti_request()
            self.assertIsNone(lti_session(request))

            # initialize the session
            lti = LTI('initial', 'any')
            self.assertTrue(lti.verify(request))

            lti = lti_session(request)
            self.assertEqual(lti.user_id(request), 'student')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('lti_provider', '0002_auto_20151231_1107'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='lticoursecontext',
            unique_together=set([('group', 'faculty_group')]),
        ),
    ]



from django.conf import settings
from django.contrib.auth import authenticate, login
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
from django.http.response import HttpResponseRedirect
from lti_provider.lti import LTI
from lti_provider.models import LTICourseContext


class LTIAuthMixin(object):
    role_type = 'any'
    request_type = 'any'

    def join_groups(self, request, lti, ctx):
        # add the user to the requested groups
        request.user.groups.add(ctx.group)
        for role in lti.user_roles(request):
            role = role.lower()
            if ('staff' in role or
                'instructor' in role or
                    'administrator' in role):
                request.user.groups.add(ctx.faculty_group)
                break

            if settings.LTI_TOOL_CONFIGURATION['allow_ta_access']:
                if 'teachingassistant' in role:
                    request.user.groups.add(ctx.faculty_group)
                    break

    def course_configuration(self, request, lti):
        # check if course is configured
        if settings.LTI_TOOL_CONFIGURATION['course_aware']:
            ctx = LTICourseContext.objects.get(
                lms_course_context=lti.course_context(request))

            # add user to the course
            self.join_groups(request, lti, ctx)

    def dispatch(self, request, *args, **kwargs):
        lti = LTI(self.request_type, self.role_type)

        # validate the user via oauth
        user = authenticate(request=request, lti=lti)
        if user is None:
            lti.clear_session(request)
            return HttpResponseRedirect(reverse('lti-fail-auth'))

        # login
        login(request, user)

        # configure course groups if requested
        try:
            self.course_configuration(request, lti)
        except (KeyError, ValueError, LTICourseContext.DoesNotExist):
            return HttpResponseRedirect(reverse('lti-course-config'))

        self.lti = lti
        return super(LTIAuthMixin, self).dispatch(request, *args, **kwargs)


class LTILoggedInMixin(object):
    role_type = 'any'
    request_type = 'any'

    def dispatch(self, request, *args, **kwargs):
        lti = LTI(self.request_type, self.role_type)

        # validate the user via oauth
        user = authenticate(request=request, lti=lti)
        if user is None:
            lti.clear_session(request)
            return HttpResponseRedirect(reverse('lti-fail-auth'))

        # login
        login(request, user)

        self.lti = lti
        return super(LTILoggedInMixin, self).dispatch(request, *args, **kwargs)



import time

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import Group
from django.http.response import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls.exceptions import NoReverseMatch
from django.utils.decorators import method_decorator
from django.views.decorators.clickjacking import xframe_options_exempt
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View, TemplateView
from lti_provider.mixins import LTIAuthMixin, LTILoggedInMixin
from lti_provider.models import LTICourseContext
from pylti.common import LTIPostMessageException, post_message


try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse


class LTIConfigView(TemplateView):
    template_name = 'lti_provider/config.xml'
    content_type = 'text/xml; charset=utf-8'

    def get_context_data(self, **kwargs):
        domain = self.request.get_host()
        launch_url = '%s://%s/%s' % (
            self.request.scheme, domain,
            settings.LTI_TOOL_CONFIGURATION.get('launch_url'))

        ctx = {
            'domain': domain,
            'launch_url': launch_url,
            'title': settings.LTI_TOOL_CONFIGURATION.get('title'),
            'description': settings.LTI_TOOL_CONFIGURATION.get('description'),
            'embed_icon_url':
                settings.LTI_TOOL_CONFIGURATION.get('embed_icon_url'),
            'embed_tool_id': settings.LTI_TOOL_CONFIGURATION.get(
                'embed_tool_id'),
            'frame_width': settings.LTI_TOOL_CONFIGURATION.get('frame_width'),
            'frame_height': settings.LTI_TOOL_CONFIGURATION.get(
                'frame_height'),
            'course_navigation': settings.LTI_TOOL_CONFIGURATION.get(
                'course_navigation'),
            'custom_fields': settings.LTI_TOOL_CONFIGURATION.get(
                'custom_fields')
        }
        return ctx


@method_decorator(xframe_options_exempt, name='dispatch')
class LTIRoutingView(LTIAuthMixin, View):
    request_type = 'initial'
    role_type = 'any'

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super(LTIRoutingView, self).dispatch(*args, **kwargs)

    def add_custom_parameters(self, url):
        if not hasattr(settings, 'LTI_EXTRA_PARAMETERS'):
            return url

        if '?' not in url:
            url += '?'
        else:
            url += '&'

        for key in settings.LTI_EXTRA_PARAMETERS:
            url += '{}={}&'.format(key, self.request.POST.get(key, ''))

        return url

    def lookup_assignment_name(self, assignment_name, pk):
        try:
            # first see if there is a matching named view
            url = reverse(assignment_name, kwargs={'pk': pk})
        except NoReverseMatch:
            # otherwise look it up.
            assignments = settings.LTI_TOOL_CONFIGURATION['assignments']
            url = assignments[assignment_name]

        return url

    def post(self, request, assignment_name=None, pk=None):
        if request.POST.get('ext_content_intended_use', '') == 'embed':
            domain = self.request.get_host()
            url = '%s://%s/%s?return_url=%s' % (
                self.request.scheme, domain,
                settings.LTI_TOOL_CONFIGURATION.get('embed_url'),
                request.POST.get('launch_presentation_return_url'))
        elif assignment_name:
            url = self.lookup_assignment_name(assignment_name, pk)
        elif request.GET.get('assignment', None) is not None:
            assignment_name = request.GET.get('assignment')
            pk = request.GET.get('pk')
            url = self.lookup_assignment_name(assignment_name, pk)
        elif settings.LTI_TOOL_CONFIGURATION.get('new_tab'):
            url = reverse('lti-landing-page')
        else:
            url = settings.LTI_TOOL_CONFIGURATION['landing_url'].format(
                self.request.scheme, self.request.get_host())

        # custom parameters can be tacked on here
        url = self.add_custom_parameters(url)

        return HttpResponseRedirect(url)


@method_decorator(xframe_options_exempt, name='dispatch')
class LTILandingPage(LTIAuthMixin, TemplateView):
    template_name = 'lti_provider/landing_page.html'

    def get_context_data(self, **kwargs):
        domain = self.request.get_host()
        url = settings.LTI_TOOL_CONFIGURATION['landing_url'].format(
            self.request.scheme, domain, self.lti.course_context(self.request))
        is_auth_ta = None
        if settings.LTI_TOOL_CONFIGURATION['allow_ta_access']:
            role = self.request.session.get('roles', '').lower()
            is_auth_ta = 'teachingassistant' in role

        return {
            'landing_url': url,
            'title': settings.LTI_TOOL_CONFIGURATION.get('title'),
            'is_instructor': self.lti.is_instructor(self.request),
            'is_administrator': self.lti.is_administrator(self.request),
            'is_auth_ta': is_auth_ta
        }


@method_decorator(xframe_options_exempt, name='dispatch')
class LTIFailAuthorization(TemplateView):
    template_name = 'lti_provider/fail_auth.html'


@method_decorator(xframe_options_exempt, name='dispatch')
class LTICourseConfigure(LTILoggedInMixin, TemplateView):
    template_name = 'lti_provider/fail_course_configuration.html'

    def get_context_data(self, **kwargs):
        return {
            'is_instructor': self.lti.is_instructor(self.request),
            'is_administrator': self.lti.is_administrator(self.request),
            'user': self.request.user,
            'lms_course': self.lti.course_context(self.request),
            'lms_course_title': self.lti.course_title(self.request),
            'sis_course_id': self.lti.sis_course_id(self.request),
            'domain': self.lti.canvas_domain(self.request)
        }


@method_decorator(xframe_options_exempt, name='dispatch')
class LTICourseEnableView(LTILoggedInMixin, View):

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(self.__class__, self).dispatch(request, *args, **kwargs)

    def post(self, *args, **kwargs):
        group_id = self.request.POST.get('group')
        faculty_group_id = self.request.POST.get('faculty_group')
        course_context = self.lti.course_context(self.request)
        title = self.lti.course_title(self.request)

        (ctx, created) = LTICourseContext.objects.get_or_create(
            group=get_object_or_404(Group, id=group_id),
            faculty_group=get_object_or_404(Group, id=faculty_group_id),
            lms_course_context=course_context)

        messages.add_message(
            self.request, messages.INFO,
            '<strong>Success!</strong> {} is connected to {}.'.format(
                title, settings.LTI_TOOL_CONFIGURATION.get('title')))

        url = reverse('lti-landing-page', args=[course_context])
        return HttpResponseRedirect(url)


class LTIPostGrade(LTIAuthMixin, View):

    def message_identifier(self):
        return '{:.0f}'.format(time.time())

    def post(self, request, *args, **kwargs):
        """
        Post grade to LTI consumer using XML

        :param: score: 0 <= score <= 1. (Score MUST be between 0 and 1)
        :return: True if post successful and score valid
        :exception: LTIPostMessageException if call failed
        """
        try:
            score = float(request.POST.get('score'))
        except ValueError:
            score = 0

        redirect_url = request.POST.get('next', '/')
        launch_url = request.POST.get('launch_url', None)

        xml = self.lti.generate_request_xml(
            self.message_identifier(), 'replaceResult',
            self.lti.lis_result_sourcedid(request), score, launch_url)

        if not post_message(
            self.lti.consumers(), self.lti.oauth_consumer_key(request),
                self.lti.lis_outcome_service_url(request), xml):

            msg = ('An error occurred while saving your score. '
                   'Please try again.')
            messages.add_message(request, messages.ERROR, msg)

            # Something went wrong, display an error.
            # Is 500 the right thing to do here?
            raise LTIPostMessageException('Post grade failed')
        else:
            msg = ('Your score was submitted. Great job!')
            messages.add_message(request, messages.INFO, msg)

            return HttpResponseRedirect(redirect_url)



from django.contrib.sessions.middleware import SessionMiddleware
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
from django.test import TestCase, RequestFactory, Client
from lti_provider.lti import LTI
from lti_provider.tests.factories import LTICourseContextFactory, \
    UserFactory, CONSUMERS, generate_lti_request, BASE_LTI_PARAMS
from lti_provider.views import LTIAuthMixin, LTIRoutingView
from pylti.common import LTI_SESSION_KEY

import xml.etree.ElementTree as ET

TEST_LTI_TOOL_CONFIGURATION = {
    'title': 'Test Application',
    'description': 'Test Description.',
    'launch_url': 'lti/',
    'embed_url': 'asset/embed/',
    'embed_icon_url': '',
    'embed_tool_id': '',
    'landing_url': '{}://{}/',
    'course_aware': True,
    'course_navigation': True,
    'new_tab': False,
    'allow_ta_access': False
}


class LTIViewTest(TestCase):

    def setUp(self):
        self.request = RequestFactory()
        self.request.COOKIES = {}
        middleware = SessionMiddleware()
        middleware.process_request(self.request)
        self.request.session.save()

        for prop, value in BASE_LTI_PARAMS.items():
            self.request.session[prop] = value

        self.lti = LTI('initial', 'any')

    def test_join_groups(self):
        mixin = LTIAuthMixin()
        ctx = LTICourseContextFactory()
        user = UserFactory()
        self.request.user = user

        mixin.join_groups(self.request, self.lti, ctx)
        self.assertTrue(user in ctx.group.user_set.all())
        self.assertTrue(user in ctx.faculty_group.user_set.all())

    def test_join_groups_student(self):
        mixin = LTIAuthMixin()
        ctx = LTICourseContextFactory()
        user = UserFactory()
        self.request.user = user

        with self.settings(
                LTI_TOOL_CONFIGURATION=TEST_LTI_TOOL_CONFIGURATION):
            self.request.session['roles'] = u'Learner'
            mixin.join_groups(self.request, self.lti, ctx)
            self.assertTrue(user in ctx.group.user_set.all())
            self.assertFalse(user in ctx.faculty_group.user_set.all())

    def test_join_groups_teachingassistant_false(self):
        mixin = LTIAuthMixin()
        ctx = LTICourseContextFactory()
        user = UserFactory()
        self.request.user = user
        lti_tool_config1 = TEST_LTI_TOOL_CONFIGURATION.copy()

        with self.settings(
                LTI_TOOL_CONFIGURATION=lti_tool_config1):
            self.request.session['roles'] = \
                u'urn:lti:role:ims/liss/TeachingAssistant'
            mixin.join_groups(self.request, self.lti, ctx)
            self.assertTrue(user in ctx.group.user_set.all())
            self.assertFalse(user in ctx.faculty_group.user_set.all())

    def test_join_groups_teachingassistant_true(self):
        mixin = LTIAuthMixin()
        ctx = LTICourseContextFactory()
        user = UserFactory()
        self.request.user = user
        lti_tool_config2 = TEST_LTI_TOOL_CONFIGURATION.copy()
        lti_tool_config2['allow_ta_access'] = True

        with self.settings(
                LTI_TOOL_CONFIGURATION=lti_tool_config2):
            self.request.session['roles'] = \
                u'urn:lti:role:ims/lis/TeachingAssistant'
            mixin.join_groups(self.request, self.lti, ctx)
            self.assertTrue(user in ctx.group.user_set.all())
            self.assertTrue(user in ctx.faculty_group.user_set.all())

    def test_launch_invalid_user(self):
        request = generate_lti_request()

        response = LTIRoutingView().dispatch(request)
        self.assertEquals(response.status_code, 302)

        self.assertEquals(response.url, reverse('lti-fail-auth'))
        self.assertFalse(request.session.get(LTI_SESSION_KEY, False))

    def test_launch_invalid_course(self):
        with self.settings(
            LTI_TOOL_CONFIGURATION=TEST_LTI_TOOL_CONFIGURATION,
                PYLTI_CONFIG={'consumers': CONSUMERS}):
            request = generate_lti_request()

            response = LTIRoutingView().dispatch(request)
            self.assertEquals(response.status_code, 302)
            self.assertEquals(response.url, reverse('lti-course-config'))
            self.assertTrue(request.session.get(LTI_SESSION_KEY, False))

    def test_launch(self):
        with self.settings(
            LTI_TOOL_CONFIGURATION=TEST_LTI_TOOL_CONFIGURATION,
            PYLTI_CONFIG={'consumers': CONSUMERS},
                LTI_EXTRA_PARAMETERS=['lti_version']):
            ctx = LTICourseContextFactory()
            request = generate_lti_request(ctx)

            view = LTIRoutingView()
            view.request = request

            response = view.dispatch(request)
            self.assertEquals(response.status_code, 302)

            landing = 'http://testserver/?lti_version=LTI-1p0&'
            self.assertEquals(
                response.url, landing.format(ctx.lms_course_context))

            self.assertIsNotNone(request.session[LTI_SESSION_KEY])
            user = request.user
            self.assertFalse(user.has_usable_password())
            self.assertEquals(user.email, 'foo@bar.com')
            self.assertEquals(user.get_full_name(), 'Foo Baz')
            self.assertTrue(user in ctx.group.user_set.all())
            self.assertTrue(user in ctx.faculty_group.user_set.all())

    def test_launch_custom_landing_page(self):
        with self.settings(
            LTI_TOOL_CONFIGURATION=TEST_LTI_TOOL_CONFIGURATION,
            PYLTI_CONFIG={'consumers': CONSUMERS},
                LTI_EXTRA_PARAMETERS=['lti_version']):
            ctx = LTICourseContextFactory()
            request = generate_lti_request(ctx, 'canvas')

            view = LTIRoutingView()
            view.request = request

            response = view.dispatch(request)
            landing = 'http://testserver/lti/landing/{}/?lti_version=LTI-1p0&'
            self.assertEquals(response.status_code, 302)
            self.assertTrue(
                response.url,
                landing.format(ctx.lms_course_context))

            self.assertIsNotNone(request.session[LTI_SESSION_KEY])
            user = request.user
            self.assertFalse(user.has_usable_password())
            self.assertEquals(user.email, 'foo@bar.com')
            self.assertEquals(user.get_full_name(), 'Foo Baz')
            self.assertTrue(user in ctx.group.user_set.all())
            self.assertTrue(user in ctx.faculty_group.user_set.all())

    def test_embed(self):
        with self.settings(PYLTI_CONFIG={'consumers': CONSUMERS},
                           LTI_EXTRA_PARAMETERS=['lti_version'],
                           LTI_TOOL_CONFIGURATION=TEST_LTI_TOOL_CONFIGURATION):
            ctx = LTICourseContextFactory()
            request = generate_lti_request(ctx, 'canvas', 'embed')

            view = LTIRoutingView()
            view.request = request

            response = view.dispatch(request)
            self.assertEquals(response.status_code, 302)
            self.assertEquals(
                response.url,
                'http://testserver/asset/embed/?return_url=/asset/'
                '&lti_version=LTI-1p0&')

            self.assertIsNotNone(request.session[LTI_SESSION_KEY])
            user = request.user
            self.assertFalse(user.has_usable_password())
            self.assertEquals(user.email, 'foo@bar.com')
            self.assertEquals(user.get_full_name(), 'Foo Baz')
            self.assertTrue(user in ctx.group.user_set.all())
            self.assertTrue(user in ctx.faculty_group.user_set.all())

    def test_course_navigation(self):
        with self.settings(
                LTI_TOOL_CONFIGURATION=TEST_LTI_TOOL_CONFIGURATION):

            client = Client()
            response = client.get('/lti/config.xml')
            config_xml = response.content.decode()

            root = ET.fromstring(config_xml)
            course_navigation_property = root.find(
                    ".//*[@name='course_navigation']")
            enabled = course_navigation_property.find(
                    "./*[@name='enabled']").text

            self.assertEqual(enabled, 'true')

    def test_course_navigation_as_dict(self):
        lti_tool_config = TEST_LTI_TOOL_CONFIGURATION
        lti_tool_config['course_navigation'] = {
            "default": "disabled",
            "enabled": "true",
            "windowTarget": "_blank"
        }
        with self.settings(
                LTI_TOOL_CONFIGURATION=lti_tool_config):

            client = Client()
            response = client.get('/lti/config.xml')
            config_xml = response.content.decode()

            root = ET.fromstring(config_xml)
            course_navigation_property = root.find(
                    ".//*[@name='course_navigation']")
            window_target = course_navigation_property.find(
                    "./*[@name='windowTarget']").text

            self.assertEqual(window_target, '_blank')

    def test_lookup_assignment_name(self):
        lti_tool_config = TEST_LTI_TOOL_CONFIGURATION
        lti_tool_config['assignments'] = {
            '1': '/assignment/1/'
        }
        with self.settings(PYLTI_CONFIG={'consumers': CONSUMERS},
                           LTI_TOOL_CONFIGURATION=lti_tool_config):

            view = LTIRoutingView()
            assignment = view.lookup_assignment_name('1', '1')
            self.assertEqual(assignment, '/assignment/1/')



from hashlib import sha1

from django.utils.encoding import force_bytes
from nameparser import HumanName
from pylti.common import LTIException
from django.contrib.auth import get_user_model

User = get_user_model()
username_field = User.USERNAME_FIELD


class LTIBackend(object):

    def create_user(self, request, lti, username):
        # create the user if necessary
        kwargs = {username_field: username, 'password': 'LTI user'}
        user = User(**kwargs)
        user.set_unusable_password()
        if username_field != 'email':
            user.email = lti.user_email(request) or ''

        name = HumanName(lti.user_fullname(request))
        user.first_name = name.first[:30]
        user.last_name = name.last[:30]

        user.save()
        return user

    def get_hashed_username(self, request, lti):
        # (http://developers.imsglobal.org/userid.html)
        # generate a username to avoid overlap with existing system usernames
        # sha1 hash result + trunc to 30 chars should result in a valid
        # username with low-ish-chance of collisions
        uid = force_bytes(lti.consumer_user_id(request))
        return sha1(uid).hexdigest()[:30]

    def get_username(self, request, lti):
        username = lti.user_identifier(request)
        if not username:
            username = self.get_hashed_username(request, lti)
        return username

    def find_user(self, request, lti):
        # find the user via lms identifier first
        kwargs = {username_field: lti.user_identifier(request)}
        user = User.objects.filter(**kwargs).first()

        # find the user via email address, if it exists
        email = lti.user_email(request)
        if user is None and email:
            user = User.objects.filter(email=email).first()

        if user is None:
            # find the user via hashed username
            username = self.get_hashed_username(request, lti)
            user = User.objects.filter(username=username).first()

        return user

    def find_or_create_user(self, request, lti):
        user = self.find_user(request, lti)
        if user is None:
            username = self.get_username(request, lti)
            user = self.create_user(request, lti, username)

        return user

    def authenticate(self, request, lti):
        try:
            lti.verify(request)
            return self.find_or_create_user(request, lti)
        except LTIException as e:
            lti.clear_session(request)
            print(e)
            return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.contrib.auth.models import Group
from django.db import models


class LTICourseContext(models.Model):
    group = models.ForeignKey(Group,
                              related_name='course_group',
                              on_delete=models.CASCADE)
    faculty_group = models.ForeignKey(Group,
                                      related_name='course_faculty_group',
                                      on_delete=models.CASCADE)
    lms_course_context = models.CharField(null=True,
                                          unique=True,
                                          max_length=255)

    class Meta:
        unique_together = (('group', 'faculty_group'),)



from django.contrib.sessions.middleware import SessionMiddleware
from django.test.client import RequestFactory
from django.test.testcases import TestCase

from lti_provider.auth import LTIBackend
from lti_provider.lti import LTI
from lti_provider.tests.factories import BASE_LTI_PARAMS, UserFactory


class LTIBackendTest(TestCase):

    def setUp(self):
        self.backend = LTIBackend()

        self.request = RequestFactory()
        self.request.COOKIES = {}
        middleware = SessionMiddleware()
        middleware.process_request(self.request)
        self.request.session.save()

        for prop, value in BASE_LTI_PARAMS.items():
            self.request.session[prop] = value

        self.lti = LTI('initial', 'any')

    def test_create_user(self):
        user = self.backend.create_user(self.request, self.lti, '12345')
        self.assertFalse(user.has_usable_password())
        self.assertEquals(user.email, 'foo@bar.com')
        self.assertEquals(user.get_full_name(), 'Foo Baz')

    def test_create_user_no_full_name(self):
        self.request.session.pop('lis_person_name_full')
        user = self.backend.create_user(self.request, self.lti, '12345')
        self.assertEquals(user.get_full_name(), 'student')

    def test_create_user_empty_full_name(self):
        self.request.session['lis_person_name_full'] = ''
        user = self.backend.create_user(self.request, self.lti, '12345')
        self.assertEquals(user.get_full_name(), 'student')

    def test_create_user_long_name(self):
        self.request.session['lis_person_name_full'] = (
            'Pneumonoultramicroscopicsilicovolcanoconiosis '
            'Supercalifragilisticexpialidocious')
        user = self.backend.create_user(self.request, self.lti, '12345')
        self.assertEquals(
            user.get_full_name(),
            'Pneumonoultramicroscopicsilico Supercalifragilisticexpialidoc')

    def test_find_or_create_user1(self):
        # via email
        user = UserFactory(email='foo@bar.com')
        self.assertEquals(
            self.backend.find_or_create_user(self.request, self.lti), user)

    def test_find_or_create_user2(self):
        # via lms username
        username = 'uni123'
        self.request.session['custom_canvas_user_login_id'] = username
        user = UserFactory(username=username)
        self.assertEquals(
            self.backend.find_or_create_user(self.request, self.lti), user)

    def test_find_or_create_user3(self):
        # via hashed username
        self.request.session['oauth_consumer_key'] = '1234567890'
        username = self.backend.get_hashed_username(self.request, self.lti)
        user = UserFactory(username=username)
        self.assertEquals(
            self.backend.find_or_create_user(self.request, self.lti), user)

    def test_find_or_create_user4(self):
        # new user
        self.request.session['oauth_consumer_key'] = '1234567890'
        user = self.backend.find_or_create_user(self.request, self.lti)
        self.assertFalse(user.has_usable_password())
        self.assertEquals(user.email, 'foo@bar.com')
        self.assertEquals(user.get_full_name(), 'Foo Baz')

        username = self.backend.get_hashed_username(self.request, self.lti)
        self.assertEquals(user.username, username)

    def test_get_user(self):
        user = UserFactory()
        self.assertIsNone(self.backend.get_user(1234))
        self.assertEquals(self.backend.get_user(user.id), user)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-10-09 12:34
from __future__ import unicode_literals

from django.db import migrations, models


class MySQLAddLMSCourseContext(migrations.AddField):

    def database_forwards(
            self, app_label, schema_editor, from_state, to_state):

        if schema_editor.connection.vendor.startswith("mysql"):
            super(MySQLAddLMSCourseContext, self).database_forwards(
                app_label, schema_editor, from_state, to_state)


class PostgresAddLMSCourseContext(migrations.AddField):

    def database_forwards(
            self, app_label, schema_editor, from_state, to_state):

        if schema_editor.connection.vendor.startswith("postgres"):
            super(PostgresAddLMSCourseContext, self).database_forwards(
                app_label, schema_editor, from_state, to_state)


class Migration(migrations.Migration):

    dependencies = [
        ('lti_provider', '0004_lticoursecontext_enable'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='lticoursecontext',
            name='enable',
        ),
        migrations.RemoveField(
            model_name='lticoursecontext',
            name='uuid',
        ),
        PostgresAddLMSCourseContext(
            model_name='lticoursecontext',
            name='lms_course_context',
            field=models.TextField(null=True, unique=True)
        ),
        MySQLAddLMSCourseContext(
            model_name='lticoursecontext',
            name='lms_course_context',
            field=models.CharField(max_length=255, null=True, unique=True)
        )
    ]
